#ifndef _DVD_DRIVE_H
#define _DVD_DRIVE_H

/** \brief Supported MMC commands.
 */
typedef enum
{
	SPC_INQUIRY = 0x12,
	MMC_READ_12 = 0xA8,
    MMC_READ_DVD_STR = 0xAD,
} MMC_COMMAND_ENUM;

typedef struct
{
	S32 sense_key;
	S32 asc;
	S32 ascq;
} REQ_SENSE_TYPE;


typedef struct
{
	U8 cmd[12];
	REQ_SENSE_TYPE *sense;
	U8 *buffer;
	S32 buflen;
} MMC_COMMAND_TYPE;

/*! \brief A structure that represents a CD/DVD-ROM drive.
 */
typedef struct dvd_drive_tag
{
	/* Device special file */
	char *device;			//!< The path to the drive (i.e.: /dev/something on Unix, x: on Windows).

	/* Data about the drive */
	char *vendor;			//!< The drive vendor.
	char *prod_id;			//!< The drive product ID.
	char *prod_rev;			//!< The drive product revision (Usually firmware version).
	char *model_string;		//!< The above three strings, joined in a single one.

	/* Device-dependent internal memory dump function */
	/*! The intended area should start where sector data is stored upon a READ command. Here we assume that sectors are
	 *  stored one after the other, as heuristics showed it is the case for the Hitachi MN103-based drives, but this model
	 *  might be changed in the future, if we get support for other drives.
	 */

    //!< A pointer to a function that is able to dump the drive's internal memory area.
    S32 (*memdump_fun_ptr) (struct dvd_drive_tag *dvd, U32 block_off, U32 block_len, U32 block_size, U8 *buf);

	U8 supported;			//!< True if the drive is a supported model, false otherwise.

	/* File descriptor & stuff used to access drive */
	HANDLE fd;			//!< The HANDLE to interact with the drive on Windows.

} DVD_DRIVE_TYPE;

extern DVD_DRIVE_TYPE *dvd_drive_new (char *device);
extern char *dvd_get_vendor (DVD_DRIVE_TYPE *dvd);
extern char *dvd_get_product_id (DVD_DRIVE_TYPE *dvd);
extern char *dvd_get_product_revision (DVD_DRIVE_TYPE *dvd);
extern char *dvd_get_model_string (DVD_DRIVE_TYPE *dvd);
extern U8 dvd_get_support_status (DVD_DRIVE_TYPE *dvd);
extern void dvd_init_command (MMC_COMMAND_TYPE *mmc, U8 *buf, S32 len, REQ_SENSE_TYPE *sense);
extern S32 dvd_execute_cmd (DVD_DRIVE_TYPE *dvd, MMC_COMMAND_TYPE *mmc, U8 ignore_errors);
extern S32 dvd_memdump (DVD_DRIVE_TYPE *dvd, U32 block_off, U32 block_len, U32 block_size, U8 *buf);
extern S32 dvd_read_sector_dummy (DVD_DRIVE_TYPE *dvd, U32 sector, REQ_SENSE_TYPE *sense);
extern S32 dvd_read_sector_streaming (DVD_DRIVE_TYPE *dvd, U32 sector, REQ_SENSE_TYPE *sense, U8 *extbuf, size_t extbufsize);
extern void *dvd_drive_destroy (DVD_DRIVE_TYPE *dvd);
extern void dvd_get_dmi(DVD_DRIVE_TYPE *dvd);
extern void dvd_get_pfi(DVD_DRIVE_TYPE *dvd);
extern void dvd_get_bca(DVD_DRIVE_TYPE *dvd);

#endif
