#include <stdio.h>
#include <time.h>
#include <windows.h>
#include "types.h"
#include "dvd_drive.h"
#include "disk.h"
#include "dumper.h"
#include "main.h"

#define USECS_PER_SEC	         1000000

#define REQ_COMMAND_LINE_PRAM    2
#define DRIVE_LABEL              1
#define USAGE                    "Usage example GCDiskDumper d: \n"

#define DELTA_EPOCH_IN_MICROSECS    11644473600000000ULL

static S32 gettimeofday(struct timeval *tv);

/* Dump GC disk to file */
int main(int argc, char *argv[])
{
    DISK_TYPE *disk = NULL;
    U8 drive_supported;

    char *type; 
    char *game_id;
    char *region;
    char *maker_id;
    char *maker;
    char *version;
    char *title;
    DISK_TYPE_ENUM typeid;
    char tmp[13];
    char *iso_out;
    DUMPER_TYPE *dmp = NULL;
    PROG_STATS_TYPE stats;

    double duration;
    long us;

    if(argc < REQ_COMMAND_LINE_PRAM)
    {
        printf("Incorrect number of args \n");
        printf(USAGE);
        return 1;
    }
    
    printf("Initializing DVD drive...");

    /* try and open drive */
    if((disk = disc_new(argv[DRIVE_LABEL])) == NULL)
    {
        printf("Failed\n");
    }
    else
    {
        /* got access to drive :-) */
        printf("OK\n");

	    drive_supported = disc_get_drive_support_status(disk);
	    printf
        (
		    "\n"
			"Drive information:\n"
			"----------------------------------------------------------------------\n"
			"Drive model........: %s\n"
			"Supported..........: %s\n", disc_get_drive_model_string (disk), drive_supported ? "Yes" : "No"
		);

		if(0 == drive_supported)
        {
		    printf
            (
			    "\n"
				"WARNING: Unsupported DVD drive model detected, falling back to Hitachi\n"
				"specific functions. If the dumping process succeeds, please report the\n"
				"drive identification strings so that it can be added to the supported\n"
				"drives list.\n"
		    );
		}

		if(0 == disc_init(disk))
        {
		    printf("failed check\n");
		}
        else
        {
		    printf("check OK\n");
		    disc_get_type (disk, &typeid, &type);
			disc_get_gameid (disk, &game_id);
			disc_get_region (disk, NULL, &region);
			disc_get_maker (disk, &maker_id, &maker);
			disc_get_version (disk, NULL, &version);
			disc_get_title (disk, &title);
			printf
            ( 
			    "\n"
				"Disc information:\n"
				"----------------------------------------------------------------------\n"
			    "Disc type..........: %s\n"
				"Game ID............: %s\n"
				"Region.............: %s\n"
				"Maker..............: %s (%s)\n"
				"Version............: %s\n"
				"Game title.........: %s\n", type, game_id, region, maker_id, maker, version, title
			);

            /* copy 8 chars from title string to tmp string */
            strncpy(tmp, title, 8);
            /* add null */
            tmp[8] = '\0'; 
            /* add file type to end of string */
            strcat (tmp,".iso");
            /* use this string as the output file name */
			iso_out = strdup(tmp);

            printf("Writing to file \"%s\" in ISO format\n\n", iso_out);

            dmp = dumper_new(disk);

            if(FALSE == dumper_set_iso_output_file(dmp, iso_out))
            {
                printf("Cannot setup new ISO output file\n");
            }
            else if(FALSE == dumper_prepare(dmp))
            {
                printf("Cannot prepare dumper\n");
            }
            else
            {
                dumper_set_progress_callback (dmp, (progress_func) progress, &stats);

                if (dumper_dump (dmp))
                {
				    printf("Dump completed successfully!\n");

                    /* work out how many seconds it took */
			        duration = stats.end_time.tv_sec - stats.start_time.tv_sec;
                    /* work out how many usec's it took */ 
			        if (stats.end_time.tv_usec >= stats.start_time.tv_usec)
                    {
				        us = stats.end_time.tv_usec - stats.start_time.tv_usec;
			        }
                    else
                    {
				        if (duration > 0)
                        {
                            /* reduce seconds count by one */
					        duration--;
                        }

				        us = USECS_PER_SEC + stats.end_time.tv_usec - stats.start_time.tv_usec;
			        }
                    
                    /* add usec to seconds count */
			        duration += ((double) us / (double) USECS_PER_SEC);
			        if (duration < 0)
                    {
				        duration = 0;
                    }

			        printf("Operation took %.2f seconds\n", duration);

                    /* get extra data */
                    printf("Reading BCA / PFI / DMI area of disk...\n");
                    dvd_get_pfi(disk->dvd);
                    dvd_get_dmi(disk->dvd);
                    dvd_get_bca(disk->dvd);
                    
                }
                else
                {
                    printf("\nDump failed!\n");
                } 

                dmp = dumper_destroy (dmp);                        
            }
        }

       disk = disc_destroy(disk);
    }
	
    return 0;
}

void progress (U8 start, U32 sectors_done, U32 total_sectors, PROG_STATS_TYPE *stats)
{
    S32 perc, i;
    double elapsed;
    double mb_done;
    double mb_done_real; 
    double mb_hour; 
    double seconds_left;
    struct timeval now;
    time_t eta;
    struct tm *etatm = NULL;
    char buf[50];
        
    if(1 == start)
    {
        gettimeofday (&(stats -> start_time));
        stats -> mb_total = (double) total_sectors * 2064 / 1024 / 1024;
        stats -> mb_total_real = (double) (total_sectors - sectors_done) * 2064 / 1024 / 1024;
        stats -> sectors_skipped = sectors_done;
    }
    else
    {
        perc = (int) (100.0 * sectors_done / total_sectors);
        gettimeofday (&now);
        elapsed = difftime (now.tv_sec, (stats -> start_time).tv_sec);
        mb_done = (double) sectors_done * 2064 / 1024 / 1024;
        mb_done_real = (double) (sectors_done - stats -> sectors_skipped) * 2064 / 1024 / 1024;
        mb_hour = mb_done_real / elapsed * 60 * 60;
        seconds_left = stats -> mb_total_real / mb_hour * 60 * 60;
        eta = (time_t) ((stats -> start_time).tv_sec + seconds_left);
                
        etatm = localtime(&eta);
                
        if(etatm != NULL)
        {
            strftime (buf, 50, "%d/%m/%Y %H:%M:%S", etatm);
        }
        else
        {
            sprintf (buf, "N/A");
        }

        printf("\r%3d ", perc);
        printf("|");
                
        for (i = 0; i < 100 / 3; i++)
        {
            if (i == perc / 3)
            {
                printf("*");
            }
            else
            {
                printf("-");
            }
        }
                
        printf("| ");
        printf("%.2lf MB/h, ETA: %s", mb_hour, buf);
        fflush(stdout);
    }

    if (sectors_done == total_sectors)
    {
        printf ("\n");
    }

    /* Save return time, in case this will be the last call */
    gettimeofday (&(stats -> end_time));

    return;
}

static S32 gettimeofday(struct timeval *tv)
{
    U64 tmpres = 0;
    FILETIME ft;

    if (tv == NULL)
    {
        return -1;
    }
    
    GetSystemTimeAsFileTime(&ft);

    tmpres |= ft.dwHighDateTime;
    tmpres <<= 32;
    tmpres |= ft.dwLowDateTime;

    /*convert into microseconds*/
    tmpres /= 10;
    
    /* converting file time to unix epoch */
    tmpres -= DELTA_EPOCH_IN_MICROSECS; 
    tv->tv_sec = (long)(tmpres / 1000000UL);
    tv->tv_usec = (long)(tmpres % 1000000UL);
   
    return 0;
}
